<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WDS checkout class.
 */
class Iconic_WDS_Checkout {
	/**
	 * Run.
	 */
	public static function run() {
		add_filter( 'woocommerce_checkout_fields', array( __CLASS__, 'checkout_fields' ), 10, 1 );
		add_action( 'woocommerce_checkout_process', array( __CLASS__, 'checkout_process' ), 10 );
	}

	/**
	 * Modify checkout fields.
	 *
	 * @param array $fields
	 *
	 * @return array
	 */
	public static function checkout_fields( $fields ) {
		if ( ! isset( $_POST['jckwds-delivery-date'] ) ) {
			return $fields;
		}

		global $jckwds;

		$checkout_fields_data = $jckwds->get_checkout_fields_data();

		if ( empty( $checkout_fields_data ) ) {
			return $fields;
		}

		$fields['jckwds'] = array();

		foreach ( $checkout_fields_data as $key => $data ) {
			$fields['jckwds'][ $key ] = array(
				'type'     => 'text',
				'label'    => $data['field_args']['label'],
				'required' => $data['field_args']['required'],
			);
		}

		return $fields;
	}

	/**
	 * Is delivery slots allowed.
	 *
	 * @return bool
	 */
	public static function is_delivery_slots_allowed() {
		if ( ! self::is_delivery_slots_allowed_for_current_shipping_method() ) {
			return (bool) apply_filters( 'iconic_wds_delivery_slots_allowed', false );
		}

		if ( ! self::is_delivery_slots_allowed_for_category() ) {
			return (bool) apply_filters( 'iconic_wds_delivery_slots_allowed', false );
		}

		if ( ! self::is_delivery_slots_allowed_for_product() ) {
			return (bool) apply_filters( 'iconic_wds_delivery_slots_allowed', false );
		}

		return (bool) apply_filters( 'iconic_wds_delivery_slots_allowed', true );
	}

	/**
	 * Check if date/time fields should be active
	 * for the current shipping method
	 *
	 * @return bool
	 */
	public static function is_delivery_slots_allowed_for_current_shipping_method() {
		global $jckwds;

		$chosen_shipping = $jckwds::get_chosen_shipping_method();
		$allowed_methods = $jckwds->settings['general_setup_shipping_methods'];

		if ( $allowed_methods && ! empty( $allowed_methods ) ) {
			if ( in_array( 'any', $allowed_methods ) ) {
				return true;
			}

			foreach ( $allowed_methods as $allowed_method ) {
				$allowed_method = str_replace( 'wc_shipping_', '', $allowed_method );

				if ( $chosen_shipping == $allowed_method ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Check if date/time should be active based on
	 * categories of products in the cart.
	 *
	 * @return bool
	 */
	public static function is_delivery_slots_allowed_for_category() {
		global $jckwds;

		$exclude_categories = $jckwds->settings['general_setup_exclude_categories'];

		if ( empty( $exclude_categories ) ) {
			return true;
		}

		$cart_contents = WC()->cart->get_cart_contents();

		foreach( $cart_contents as $cart_key => $cart_item ) {
			if ( empty( $cart_item['data'] ) ) {
				continue;
			}

			$product = $cart_item['data'];
			$product_parent_id = $cart_item['data']->get_parent_id();

			if ( $product_parent_id ) {
				$product = wc_get_product( $product_parent_id );
			}

			$category_ids = $product->get_category_ids();

			if ( empty( $category_ids ) ) {
				continue;
			}

			$compare = array_intersect($exclude_categories, $category_ids);

			if ( empty( $compare ) ) {
				continue;
			}

			return false;
		}

		return true;
	}

	/**
	 * Check if date/time should be active based on
	 * products in the cart.
	 *
	 * @return bool
	 */
	public static function is_delivery_slots_allowed_for_product() {
		global $jckwds;

		$exclude_products = $jckwds->settings['general_setup_exclude_products'];

		if ( ! is_array( $exclude_products ) || empty( $exclude_products ) ) {
			return true;
		}

		$cart_contents = WC()->cart->get_cart_contents();

		foreach( $cart_contents as $cart_key => $cart_item ) {
			if ( empty( $cart_item['data'] ) ) {
				continue;
			}

			if ( ! in_array( $cart_item['product_id'], $exclude_products ) ) {
				continue;
			}

			return false;
		}

		return true;
	}

	/**
	 * Validate checkout fields.
	 */
	public static function checkout_process() {
		if ( ! empty( $_POST['iconic-wds-fields-hidden'] ) ) {
			return;
		}

		global $jckwds;

		if ( ! $jckwds->settings['timesettings_timesettings_setup_enable'] ) {
			return;
		}

		if ( $jckwds->settings['timesettings_timesettings_setup_mandatory'] && empty( $_POST['jckwds-delivery-time'] ) ) {
			$label = '<strong>' . Iconic_WDS_Helpers::get_label( 'time_slot' ) . '</strong>';
			wc_add_notice( sprintf( __( '%s is a required field.', 'woocommerce' ), $label ), 'error' );
		}
	}
}